﻿/// Copyright (c) 2011 Andrey Veselov. All rights reserved.
/// WebSite: http://andrey.moveax.ru 
/// Email: andrey@moveax.ru
/// This source is subject to the Microsoft Public License (Ms-PL).

namespace MVCDemo.Attributes.Validation
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel.DataAnnotations;
    using System.Text.RegularExpressions;
    using System.Web.Mvc;

    [AttributeUsage(AttributeTargets.Property, AllowMultiple = false, Inherited = true)]
    public class BlockHtmlAttribute : ValidationAttribute, IClientValidatable
    {
        private const string _hasTags =
            @"^(&\u0023)|^(<[/!a-zA-Z])|^((.(?!(?:(&\u0023)|(<[/!a-zA-Z]))))*)$";

        public bool ValidateOnServer { get; set; }

        public BlockHtmlAttribute() :
            base("{0} has invalid symbols in it.")
        {
            this.ValidateOnServer = false;
        }

        #region ValidationAttribute overrides

        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            if (!this.ValidateOnServer) {
                return ValidationResult.Success;
            }

            string stringValue = value as string;
            if (stringValue == null) {
                return ValidationResult.Success;
            }

            var regex = new Regex(BlockHtmlAttribute._hasTags);
            Match match = regex.Match(stringValue);

            if ((match.Success && (match.Index == 0)) &&
                (match.Length == stringValue.Length)) {
                return ValidationResult.Success;
            }

            return new ValidationResult(
                    this.FormatErrorMessage(validationContext.DisplayName));
        }

        #endregion

        #region IClientValidatable Members

        public IEnumerable<ModelClientValidationRule> GetClientValidationRules(
            ModelMetadata metadata, ControllerContext context)
        {
            var rule = new ModelClientValidationRule() {
                ValidationType = "blockhtml",
                ErrorMessage = this.FormatErrorMessage(metadata.DisplayName)
            };

            yield return rule;
        }

        #endregion
    }
}